﻿package de.davidochmann.utilities.dogesture.logic 
{
	/**
	 * @author dochmann
	 */

	import flash.events.Event;
	import flash.utils.Timer;	 
	import flash.events.TimerEvent;	
	import flash.events.MouseEvent;	
	import flash.display.DisplayObject;	
	import flash.events.EventDispatcher;		
	import de.davidochmann.utilities.dogesture.move.Move;
	import de.davidochmann.utilities.dogesture.stack.Stack;
	import de.davidochmann.utilities.dogesture.events.GestureEvent;	 
		
	public class Logic extends EventDispatcher
	{	
		private var mStack:Stack;
		private var mSurface:DisplayObject;
		private var mTimeout:uint;
		private var mObject:Object;
		private var mComplete:Boolean;
		private var mTimer:Timer;	
		private var mCurrent:Move;
		private var mCompleteX:Boolean;
		private var mCompleteY:Boolean;
		private var mStartX:int;
		private var mStartY:int;
		private var mMoveX:int;
		private var mMoveY:int;
		private var mCurrentX:int;
		private var mCurrentY:int;
		private var mActive:Boolean;



		public function Logic(pStack:Stack,
							  pSurface:DisplayObject,
							  pTimeout:uint,
							  pObject:Object):void
		{
			setStack(pStack);
			setSurface(pSurface);
			setTimeout(pTimeout);
			setObject(pObject);
		}
		
		
		
		/*
		 * PUBLIC FUNCTIONS
		 */	
			
		public function start():void
		{
			initComplete();
			initActive();
			initSurfaceEvents();
			initTimer();
		}

		public function stop():void
		{
			if(mObject == null)
			{
				mSurface.removeEventListener(MouseEvent.MOUSE_DOWN, surfaceMouseDownHandler);
				mSurface.removeEventListener(MouseEvent.MOUSE_UP  , surfaceMouseUpHandler);
				mSurface.removeEventListener(MouseEvent.MOUSE_MOVE, surfaceMouseMoveHandler);
			}
			else 
				mSurface.removeEventListener(Event.ENTER_FRAME, surfaceEnterFrameHandler);
				
			mActive = false;
		}

		public function active():Boolean
		{
			return mActive;
		}

		public function complete():Boolean
		{
			return mComplete;
		}

		
		
		/*
		 * GETTER FUNCTIONS
		 */
		
		public function getStack():Stack
		{
			return mStack;
		}

		public function getSurface():DisplayObject
		{
			return mSurface;
		}

		public function getTimeout():uint
		{
			return mTimeout;
		}

		public function getObject():Object
		{
			return mObject;
		}

		
		
		/*
		 * SETTER FUNCTIONS
		 */
		
		public function setStack(pStack:Stack):void
		{
			mStack = pStack;
		}
		
		public function setSurface(pSurface:DisplayObject):void
		{
			mSurface = pSurface;		
		}

		public function setTimeout(pTimeout:uint):void
		{
			mTimeout = pTimeout;
		}

		public function setObject(pObject:Object):void
		{
			mObject = pObject;
		}

		
		
		/*
		 * PRIVATE FUNCTIONS
		 */
		
		private function initComplete():void
		{
			mComplete = false;
		}

		private function initActive():void
		{
			mActive = true;
		}


		
		/*
		 * SURFACE EVENTS
		 */

		private function initSurfaceEvents():void
		{
			if(mObject == null)
			{
				mSurface.addEventListener(MouseEvent.MOUSE_DOWN, surfaceMouseDownHandler);
				mSurface.addEventListener(MouseEvent.MOUSE_UP  , surfaceMouseUpHandler);
			}
			else
			{
				mSurface.addEventListener(Event.ENTER_FRAME, surfaceEnterFrameHandler);
				setStart(mObject.x, mObject.y);				
			}
		}
				
		private function surfaceMouseDownHandler(e:MouseEvent):void
		{		
			if(mTimer != null) mTimer.start();
								
			setStart(e.localX, e.localY);
			mSurface.addEventListener(MouseEvent.MOUSE_MOVE, surfaceMouseMoveHandler);
		}

		private function surfaceMouseUpHandler(e:MouseEvent):void
		{	
			if(mComplete) dispatchEvent(new GestureEvent(GestureEvent.GESTURE_COMPLETE, mComplete, mStack.index(), mStack.list())); 
			mSurface.removeEventListener(MouseEvent.MOUSE_MOVE, surfaceMouseMoveHandler);
			
			mActive = false;
			mStack.reset();		
		}
					
		private function setStart(pX:Number, pY:Number):void
		{
			mActive   = true;
			mComplete = false;
			mCurrent  = mStack.active();
			
			mCurrent.setStartX(pX);
			mCurrent.setStartY(pY);
		}

		private function surfaceMouseMoveHandler(e:MouseEvent):void
		{
			e.updateAfterEvent();
			moveHandler(e.localX, e.localY);
		}
		
		private function surfaceEnterFrameHandler(e:Event):void
		{
			moveHandler(mObject.x, mObject.y);			
		}

		private function moveHandler(pX:Number, pY:Number):void
		{
			mCompleteX = false;
			mCompleteY = false;
			mStartX	   = mStack.active().getStartX();
			mStartY	   = mStack.active().getStartY();
			mMoveX	   = mStack.active().getPositionX();
			mMoveY	   = mStack.active().getPositionY();
			mCurrentX  = pX;
			mCurrentY  = pY;


			mCompleteX = (mMoveX != 0) ? ((mMoveX > 0) ? (mCurrentX - mStartX > mMoveX) : (mCurrentX - mStartX < mMoveX)) : true;
			mCompleteY = (mMoveY != 0) ? ((mMoveY > 0) ? (mCurrentY - mStartY > mMoveY) : (mCurrentY - mStartY < mMoveY)) : true;
					
			mComplete  = (mCompleteX && mCompleteY);
			
			
			if(mComplete) mStack.next();
				
				
			if(mComplete && !mStack.complete())
			{
				mComplete = false;
				mCurrent  = mStack.active();
				
				mCurrent.setStartX(pX);
				mCurrent.setStartY(pY);
				
				dispatchEvent(new GestureEvent(GestureEvent.MOVE_COMPLETE, mComplete, mStack.index(), mStack.list()));
			}
			else if(mComplete && mStack.complete())
			{
				if(mTimer != null) mTimer.stop();
				
				if(mObject == null) mSurface.removeEventListener(MouseEvent.MOUSE_MOVE, surfaceMouseMoveHandler);
				else
				{
					mSurface.removeEventListener(Event.ENTER_FRAME, surfaceEnterFrameHandler);
					dispatchEvent(new GestureEvent(GestureEvent.GESTURE_COMPLETE, mComplete, mStack.index(), mStack.list())); 
					mActive = false;
					mStack.reset();										
				}
				
				dispatchEvent(new GestureEvent(GestureEvent.MOVE_COMPLETE, mComplete, mStack.index(), mStack.list()));
			}	
		}
		
		
		
		/*
		 * TIMER FUNCTIONS
		 */
		
		private function initTimer():void
		{
			if(mTimeout == 0) return;
			
			mTimer = new Timer(mTimeout, 0);
			mTimer.addEventListener(TimerEvent.TIMER, timerTimerHandler);	
		}
		
		private function timerTimerHandler(e:TimerEvent):void
		{			
			mTimer.stop();
			stop();
		}
	}
}
